Package: PROMISE
Type: Package
Title: PRojection Onto the Most Interesting Statistical Evidence
Description: A general tool to identify genomic features with a
        specific biologically interesting pattern of associations with
        multiple endpoint variables as described in Pounds et. al.
        (2009) Bioinformatics 25: 2013-2019
Version: 1.62.0
Date: 2014-6-24
Author: Stan Pounds <stanley.pounds@stjude.org>, Xueyuan Cao
        <xueyuan.cao@stjude.org>
Maintainer: Stan Pounds <stanley.pounds@stjude.org>, Xueyuan Cao
 <xueyuan.cao@stjude.org>
Depends: R (>= 3.1.0), Biobase, GSEABase
Imports: Biobase, GSEABase, stats
License: GPL (>= 2)
LazyLoad: yes
biocViews: Microarray, OneChannel, MultipleComparison, GeneExpression
git_url: https://git.bioconductor.org/packages/PROMISE
git_branch: RELEASE_3_22
git_last_commit: 7bf04c1
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.22
Date/Publication: 2025-10-29
NeedsCompilation: no
Packaged: 2025-10-30 04:58:51 UTC; biocbuild
Built: R 4.5.1; ; 2025-10-30 12:32:05 UTC; unix

export("PROMISE", "avg.abs.genestat", "jung.rstat", "spearman.rstat", "promise.genestat")

importMethodsFrom(Biobase, exprs, pData, phenoData)
importMethodsFrom(GSEABase, geneIds, setName)
importFrom(stats, sd)

  promise:  bytecode optimisation using staticness assertions.

This is a module for applying some simple optimisations to function bytecode.
By promising that a function doesn't do certain things at run-time, it's
possible to apply optimisations that are not legal in the general case.

As a simple example, it's possible to promise that a function doesn't modify
(or care if anyone else modifies) any builtin functions by decorating it thus:

    @promise.constant(__builtins__)
    def function():
        ...

Such a promise will allow the builtins to be stored as direct object references
in the function bytecode, avoiding name lookups during function execution.

As another example, it's possible to promise that a function is pure; i.e. that
it's a simple algorithm for mapping input values to an output value:

    @promise.pure()
    def calculate(a,b):
        return 2*a*a + 3*b + 7

If a pure function is then used by another function as a constant, it can be
directly inlined into the bytecode to avoid the overhead of a function call:

    @promise.constant(("calculate",))
    def aggregate(pairs):
        #  calculate() is a pure constant, so it will be inlined here.
        return sum(calculate(a,b) for (a,b) in pairs)

The currently available promises are:

    * invariant(names):  promise that variables having the given names will
                         not change value during execution of the function.

    * constant(names):   promise that variables having the given names will
                         always refer to the same object, across all calls
                         to the function.

    * pure():   promise that the function is a transparent mapping from inputs
                to outputs; this opens up the possibility of inling it directly
                into other functions.

    * sensible():   promise that the function is "sensibly behaved".  All
                    builtins and module-level functions are considered
                    constant; all other module-level names are considered
                    invariant.

Promise is built on Noam Raphael's fantastic "byteplay" module; since the
official byteplay distribution doesn't support Python 2.6, a local version with
appropriate patches is included with promise.

`PROMISE` <-
function(exprSet, geneSet=NULL, promise.pattern, strat.var=NULL, proj0=FALSE, seed=13, nbperm=FALSE, max.ntail=100, nperms=10000)

{
     # extract data components from objects
     ph.pattern<-promise.pattern
     endpt.data<-pData(phenoData(exprSet))
     array.data<-exprs(exprSet)
     if (is.data.frame(array.data)) array.data<-as.matrix(array.data)
     if (!is.numeric(array.data))
     {
        print ("ERROR: The array data is not numeric")
       return()
     } 
     if (!is.null(geneSet)) {
          geneset.data<-NULL
          for (i in 1:length(geneSet)){
              tt<-geneSet[i][[1]]
              this.name<-unlist(geneIds(tt))
              this.set<-setName(tt)
              geneset.data<-rbind.data.frame(geneset.data, cbind.data.frame(featureID=as.character(this.name),
                                          setID=rep(as.character(this.set), length(this.name))))
          }
      }
     #geneset.data<-geneset.data[order(geneset.data[, 1]), ]
     # Match array data and endpttype data by subject identifiers
     array.names<-dimnames(array.data)[[2]]
     endpt.names<-dimnames(endpt.data)[[1]]

     array.inc<-is.element(array.names,endpt.names)
     endpt.inc<-is.element(endpt.names,array.names)
     nsamp.inc<-sum(array.inc)
     message(paste("No. of Matching array ids in endpt data and array data:",nsamp.inc))
     
     if (nsamp.inc<=4)
     {
        message(paste("Array IDs in endpt data:"))
        message(endpt.names)
        message("Array IDs in array data:")
        message(array.names)
        stop("Error: Unable to match array identifiers in endpt data and array data.")
     }
     
          
     # Subset on array data and phenotype data with matching subject IDs
     Y<-array.data[,array.inc]
     #Y<-Y[order(dimnames(Y)[[1]]), ]
     X<-endpt.data[endpt.inc,]
     array.names<-array.names[array.inc]
     endpt.names<-endpt.names[endpt.inc]

     array.ord<-order(array.names)
     endpt.ord<-order(endpt.names)

     Y<-Y[,array.ord]
     X<-X[endpt.ord,]
     Y<-as.matrix(Y)

     array.names<-array.names[array.ord]
     endpt.names<-endpt.names[endpt.ord]

     if (any(array.names!=endpt.names))
     {
        name.mtx<-cbind(endpt.arrayid=endpt.names,array.arrayid=array.names)
        message("----------------- IDs ------------------")
        message(name.mtx)
        stop("Error: Unable to align IDs in array data and endpt data")
     }
     
     endpt.endptcol<-is.element(dimnames(X)[[2]],unique(unlist(strsplit(c(as.character(ph.pattern$endpt.vars),
                                         strat.var), ","))) )
     phtype<-X[,endpt.endptcol]
     names(phtype)<-dimnames(endpt.data)[[2]][endpt.endptcol]
     if (!is.numeric(phtype[, dimnames(phtype)[[2]]!=strat.var])&!is.data.frame(phtype[, dimnames(phtype)[[2]]!=strat.var]))
     {
        print ("ERROR: The clinical data is not numeric or data frame")
       return()
     } 

     if (is.vector(phtype)) phtype<-matrix(phtype,length(phtype),1)
     
     # Now extract probe set names
     probes<-dimnames(array.data)[[1]]
     nprobes<-length(probes)
     
     if (nbperm)
     {      
       if (!is.null(geneSet)) 
       {

         #Now extract probe and gene set names
         gs.probes<-geneset.data[,1]
         gs.sets<-geneset.data[,2]

         uset<-unique(gs.sets)
         nset<-length(uset)
        
         # Now compute results for observed data
         message(paste("Computing Stats for Observed Data:",date()))
         message(paste("Computing gene-specific statistics:",date()))
         if (is.null(strat.var)) strat<-NULL  
         else  strat<-phtype[,strat.var] 
         gene.res<-promise.genestat(Y,phtype,ph.pattern, strat, proj0=proj0)

         m.nph<-dim(gene.res)
         gene.pvals<-matrix(0,m.nph[1],m.nph[2])
         
         message(paste("Computing gene-set enrichment statistics:",date()))
         enrich.res<-avg.abs.genestat(gene.res,probes, geneset.data)
     
         ngs.nph<-dim(enrich.res)
         enrich.pvals<-matrix(0,ngs.nph[1],ngs.nph[2])
     
         #set seed for sampling subjects
         message(paste("Set seed: seed = ",seed))
         set.seed(seed)
         
         probe.keep<-rep(T,nprobes)
         probe.done<-rep(F,nprobes)
         probe.ntail<-matrix(0,m.nph[1],m.nph[2])
         probe.apt.ntail<-rep(0,m.nph[1])
         
         set.keep<-rep(T,ngs.nph[1])
         set.done<-rep(F,ngs.nph[1])
         set.ntail<-matrix(0,ngs.nph[1],ngs.nph[2])
         set.apt.ntail<-rep(0,ngs.nph[1])

         set.probe.ntail<-rep(NA,nprobes)
         set.probe.keep<-rep(NA,nprobes)
         set.probe.keep[is.element(probes, geneset.data[, 1])]<-T
         set.probe.done<-rep(F,nprobes)
         
         num.perms<-rep(NA,nprobes)
         set.perms<-rep(NA,nset)
         
         i<-0
         while((i<nperms)&&(any(probe.keep)))
         {
           i<-i+1
           if (is.null(strat.var)) perm.ind<-sample(nsamp.inc,replace=FALSE)
           else
           {
             strat<-X[,strat.var]
             ustrat<-sort(unique(strat))
             nstrat<-length(ustrat)
             ind<-length(strat)
             for (j in 1:nstrat)
             {
               this.strat<-(strat == ustrat[j])
               perm.ind[this.strat]<-sample((1:ind)[this.strat],replace=FALSE)
             }
           }
           perm.phtype<-phtype[perm.ind,]
           
           if (is.null(strat.var))  perm.strat<-NULL 
           else  perm.strat<- perm.phtype[,strat.var]
                    
           gene.temp<-promise.genestat(Y[probe.keep, ],perm.phtype, ph.pattern, perm.strat, proj0=proj0)
           set.probe.keep2<-set.probe.keep[!is.na(set.probe.keep)]
           enrich.temp<-avg.abs.genestat(gene.temp,probes[probe.keep],geneset.data[set.probe.keep2, ])
           
           set.ntail[set.keep,]<-set.ntail[set.keep,] + (abs(enrich.temp)>=abs(enrich.res[set.keep, ]))
           set.apt.ntail[set.keep]<-set.ntail[set.keep, ngs.nph[2]]
           
           probe.ntail[probe.keep,]<-probe.ntail[probe.keep,]+(abs(gene.temp)>=abs(gene.res[probe.keep,]))
           probe.apt.ntail[probe.keep]<- probe.ntail[probe.keep, m.nph[2]]
           
           for (j in 1:sum(set.keep)){ 
             set.probe.ntail[!is.na(set.probe.keep)][is.element(probes[!is.na(set.probe.keep)], geneset.data[set.probe.keep2,][geneset.data[set.probe.keep2, 2]==uset[set.keep][j], 1]) ] <-
               set.apt.ntail[set.keep][j]  
           }
           
           probe.done[probe.keep]<-(probe.apt.ntail[probe.keep]>=max.ntail & set.probe.ntail[probe.keep] >= max.ntail) |  
                                   (probe.apt.ntail[probe.keep]>=max.ntail& is.na(set.probe.keep[probe.keep]))  
           set.probe.done[probe.keep & !is.na(set.probe.keep)]<-probe.done[probe.keep & !is.na(set.probe.keep)]
           
           for (j in 1:sum(set.keep)){
               set.done[set.keep][j] <- set.apt.ntail[set.keep][j]>=max.ntail & 
                 min(probe.apt.ntail[!is.na(set.probe.keep)][is.element(probes[!is.na(set.probe.keep)], geneset.data[set.probe.keep2,][geneset.data[set.probe.keep2, 2]== uset[set.keep][j], 1])])>=max.ntail
           }
           
           num.perms[probe.done&probe.keep]<-i
           set.perms[set.done&set.keep]<-i
           gene.pvals[probe.done&probe.keep, ]<-probe.ntail[probe.done&probe.keep,]/i

           enrich.pvals[set.done&set.keep, ] <-set.ntail[set.done&set.keep, ]/i  
           probe.keep[probe.done]<-F
           set.probe.keep[!is.na(set.probe.keep)& set.probe.done]<-F
           set.keep[set.done]<-F
         }
         
         if (any(probe.keep))
         {
           gene.pvals[probe.keep,]<-probe.ntail[probe.keep,]/nperms
           num.perms[probe.keep]<-nperms
         }
                  
         if (any(set.keep))
         {
           enrich.pvals[set.keep,]<-set.ntail[set.keep,]/nperms
           set.perms[set.keep]<-nperms
         }
         dimnames(gene.pvals)[[2]]<-gsub("stat", "perm.p", dimnames(gene.res)[[2]])
         dimnames(enrich.pvals)[[2]]<-gsub("stat", "perm.p", dimnames(enrich.res)[[2]])
 
         generes.tab<-cbind.data.frame(probeid=probes,
                                       gene.res,
                                       gene.pvals,
                                       nperms=num.perms)
         
         enrich.tab<-cbind.data.frame(setid=uset,
                                      enrich.res,
                                      enrich.pvals,
                                      nperms=set.perms)
         
         message(paste("Finished at: ",date()))
         
         res<-list(generes=generes.tab, setres=enrich.tab)
       }
       
       
       if (is.null(geneSet)) 
       {
         
         # Now compute results for observed data
         message(paste("Computing Stats for Observed Data:",date()))
         message(paste("Computing gene-specific statistics:",date()))
         if (is.null(strat.var)) strat<-NULL  
         else  strat<-phtype[,strat.var] 
         gene.res<-promise.genestat(Y,phtype,ph.pattern, strat, proj0=proj0)
         
         m.nph<-dim(gene.res)
         gene.pvals<-matrix(0,m.nph[1],m.nph[2])
         
         #set seed for sampling subjects
         message(paste("Set seed: seed = ",seed))
         set.seed(seed)
         
         probe.keep<-rep(T,nprobes)
         probe.done<-rep(F,nprobes)
         probe.ntail<-matrix(0,m.nph[1],m.nph[2])
         probe.apt.ntail<-rep(0,m.nph[1])
         num.perms<-rep(NA,nprobes)
         
         i<-0
         while((i<nperms)&&(any(probe.keep)))
         {
           i<-i+1
           if (is.null(strat.var)) perm.ind<-sample(nsamp.inc,replace=FALSE)
           else
           {
             strat<-X[,strat.var]
             ustrat<-sort(unique(strat))
             nstrat<-length(ustrat)
             ind<-length(strat)
             for (j in 1:nstrat)
             {
               this.strat<-(strat == ustrat[j])
               perm.ind[this.strat]<-sample((1:ind)[this.strat],replace=FALSE)
             }
           }
           perm.phtype<-phtype[perm.ind,]
           
           if (is.null(strat.var))  perm.strat<-NULL 
           else  perm.strat<- perm.phtype[,strat.var]
           
           gene.temp<-promise.genestat(Y[probe.keep, ], perm.phtype, ph.pattern, perm.strat, proj0=proj0)         
           probe.ntail[probe.keep,]<-probe.ntail[probe.keep,]+(abs(gene.temp)>=abs(gene.res[probe.keep,]))
           probe.apt.ntail[probe.keep]<- probe.ntail[probe.keep, m.nph[2]]
           
           probe.done[probe.keep]<-probe.apt.ntail[probe.keep]==max.ntail         
           num.perms[probe.done&probe.keep]<-i
           gene.pvals[probe.done&probe.keep,]<-probe.ntail[probe.done&probe.keep,]/i
           probe.keep[probe.done]<-F
         }
         
         if (any(probe.keep))
         {
           gene.pvals[probe.keep,]<-probe.ntail[probe.keep,]/nperms
           num.perms[probe.keep]<-nperms
         }
         dimnames(gene.pvals)[[2]]<-gsub("stat", "perm.p", dimnames(gene.res)[[2]])
         
         generes.tab<-cbind.data.frame(probeid=probes,
                                       gene.res,
                                       gene.pvals,
                                       nperms=num.perms)
         
         message(paste("Finished at: ",date()))
         
         res<-list(generes=generes.tab, setres=NULL)
       }
     } # end of nbperm 
     
     if (!nbperm)
     {
       if (!is.null(geneSet)) 
       {
         
         #Now extract probe and gene set names
         gs.probes<-geneset.data[,1]
         gs.sets<-geneset.data[,2]
         
         uset<-unique(gs.sets)
         nset<-length(uset)
         
         # Now compute results for observed data
         message(paste("Computing Stats for Observed Data:",date()))
         message(paste("Computing gene-specific statistics:",date()))
         if (is.null(strat.var)) strat<-NULL  
         else  strat<-phtype[,strat.var] 
         gene.res<-promise.genestat(Y,phtype,ph.pattern, strat, proj0=proj0)
         
         m.nph<-dim(gene.res)
         gene.pvals<-matrix(0,m.nph[1],m.nph[2])
         
         message(paste("Computing gene-set enrichment statistics:",date()))
         enrich.res<-avg.abs.genestat(gene.res,probes, geneset.data)
         
         ngs.nph<-dim(enrich.res)
         enrich.pvals<-matrix(0,ngs.nph[1],ngs.nph[2])
         
         #set seed for sampling subjects
         message(paste("Set seed: seed = ",seed))
         set.seed(seed)
         
         perms<-matrix(NA,nperms,nsamp.inc)
         if (is.null(strat.var)) for (i in 1:nperms) perms[i,]<-sample(nsamp.inc,replace=FALSE)
         else
         {
           strat<-endpt.data[,strat.var]
           ustrat<-sort(unique(strat))
           nstrat<-length(ustrat)
           ind<-length(strat)
           for (j in 1:nstrat)
           {
             this.strat<-(strat == ustrat[j])
             for (i in 1:nperms) perms[i,this.strat]<-sample((1:ind)[this.strat],replace=FALSE)
           }
         }
         
         message(paste("Computing Stats for Permuted Data: ",date()))    
         for (i in 1:nperms)
         {
           perm.ind<-unlist(perms[i,])
           perm.phtype<-phtype[perm.ind,]
           if (is.null(strat.var))  perm.strat<-NULL 
           else  perm.strat<- perm.phtype[,strat.var]
           
           gene.temp<-promise.genestat(Y,perm.phtype,ph.pattern, perm.strat, proj0=proj0)
           enrich.temp<-avg.abs.genestat(gene.temp,probes,geneset.data)
           
           enrich.pvals<-enrich.pvals+(abs(enrich.temp)>=abs(enrich.res))
           gene.pvals<-gene.pvals+(abs(gene.temp)>=abs(gene.res))
         }
         
         message(paste("Generating Result List: ",date()))
         gene.pvals<-gene.pvals/nperms
         enrich.pvals<-enrich.pvals/nperms

         dimnames(gene.pvals)[[2]]<-gsub("stat", "perm.p", dimnames(gene.res)[[2]])
         dimnames(enrich.pvals)[[2]]<-gsub("stat", "perm.p", dimnames(enrich.res)[[2]])
         
         generes.tab<-cbind.data.frame(probeid=probes,
                                       gene.res,
                                       gene.pvals)
         
         enrich.tab<-cbind.data.frame(setid=uset,
                                      enrich.res,
                                      enrich.pvals)
         
         message(paste("Finished at: ",date()))
         
         res<-list(generes=generes.tab, setres=enrich.tab)
       }
       
       if (is.null(geneSet))
       {
         # Now compute results for observed data
         message(paste("Computing Stats for Observed Data:",date()))
         message(paste("Computing gene-specific statistics:",date()))
         if (is.null(strat.var)) strat<-NULL  
         else  strat<-phtype[,strat.var] 
         gene.res<-promise.genestat(Y,phtype,ph.pattern, strat, proj0=proj0)
         
         m.nph<-dim(gene.res)
         gene.pvals<-matrix(0,m.nph[1],m.nph[2])
         
         #set seed for sampling subjects
         message(paste("Set seed: seed = ",seed))
         set.seed(seed)
         
         perms<-matrix(NA,nperms,nsamp.inc)
         if (is.null(strat.var)) for (i in 1:nperms) perms[i,]<-sample(nsamp.inc,replace=FALSE)
         else
         {
           strat<-endpt.data[,strat.var]
           ustrat<-sort(unique(strat))
           nstrat<-length(ustrat)
           ind<-length(strat)
           for (j in 1:nstrat)
           {
             this.strat<-(strat == ustrat[j])
             for (i in 1:nperms) perms[i,this.strat]<-sample((1:ind)[this.strat],replace=FALSE)
           }
         }
         
         message(paste("Computing Stats for Permuted Data: ",date()))    
         for (i in 1:nperms)
         {
           perm.ind<-unlist(perms[i,])
           perm.phtype<-phtype[perm.ind,]
           if (is.null(strat.var))  perm.strat<-NULL 
           else  perm.strat<- perm.phtype[,strat.var]
           
           gene.temp<-promise.genestat(Y,perm.phtype,ph.pattern, perm.strat, proj0=proj0)
           gene.pvals<-gene.pvals+(abs(gene.temp)>=abs(gene.res))
         }
         
         message(paste("Generating Result List: ",date()))
         
         gene.pvals<-gene.pvals/nperms
         dimnames(gene.pvals)[[2]]<-gsub("stat", "perm.p", dimnames(gene.res)[[2]])
         
         generes.tab<-cbind.data.frame(probeid=probes,
                                       gene.res,
                                       gene.pvals)
         message(paste("Finished at: ",date()))     
         res<-list(generes=generes.tab, setres=NULL)
       }       
       
     }

     return(res)
}

`avg.abs.genestat` <-
function(gene.res,probes,GS.data)
#gene.res: individual gene result
#probes: probeid
#GS.data: gene set definition data with probe and set variable
{
    ugs<-unique(GS.data[,2])
    ngs<-length(ugs)
    nph<-dim(gene.res)[2]
    res<-matrix(NA,ngs,nph)
    A<-abs(gene.res)
    for (i in 1:ngs)
    {
       gs<-is.element(GS.data[,2],ugs[i])
       in.gs<-is.element(probes,GS.data[gs,1])
       if (sum(in.gs)==1) res[i,] <-mean(A[in.gs,])
       else  res[i,]<-colMeans(A[in.gs,])
    }
    colnames(res)<-colnames(A)
    return(res)
}

`spearman.rstat` <-
function(Y,  x,  strat=NULL)
# Y: Array data matrix, subjects in columns, variables in rows
# x: phenotype vector, entries for subjects, same order as columns of Y
# strat=NULL: stratification vector, entries for subjects, same order as columns of Y
{
  if (is.null(strat)) strat<-rep(1, length(x))
  # Some initialization
	ustrat<-unique(strat)
	nstrat<-length(ustrat)
	stat.sum<-0
	sum.n<-0

  # Loop over stratum
	for (i in 1:nstrat)
	{
    this.strat<-(strat==ustrat[i])
		this.x<-x[this.strat]
		this.Y<-Y[,this.strat]
    this.n<-sum(this.strat)

       this.x<-unlist(this.x)
       this.x.miss<-is.na(this.x)
       this.x<-this.x[!this.x.miss]
       this.Y<-this.Y[,!this.x.miss]

       r<-rank(this.x)
       r.mn<-mean(r)
       r.sd<-sqrt(var(r, na.rm=T))
       r.cnt<-(r-r.mn)/r.sd
       r.cnt[r.sd==0]<-0

       # Now identify missing Y
       this.Y.miss<-is.na(this.Y)

       # Now rank Y
       tR<-apply(this.Y,1,rank)
       R<-t(tR)
       R[this.Y.miss]<-NA
       n<-rowSums(!this.Y.miss)
       R.mn<-rowMeans(R,na.rm=TRUE)
       R.dev<-(R-R.mn)
       R.sd<-sqrt(rowMeans(R.dev^2,na.rm=TRUE))
       R.cnt<-(R-R.mn)/R.sd
       R.cnt[this.Y.miss]<-0
       R.cnt[R.sd==0]<-0
       res<-(R.cnt%*%r.cnt)/n
       res<-as.vector(res)
       
		stat.sum<-stat.sum+this.n*res
		sum.n<-sum.n+this.n
	}
  stat.sum<-as.vector(stat.sum/sum.n)
	return(stat.sum)
}

`promise.genestat` <-
function(Y,ph.data,ph.pattern, strat=NULL, proj0=FALSE)

{
   m<-dim(Y)[1]
   res<-0
   nph<-dim(ph.pattern)[1]
   sum.wgt<-0
   ph.stats<-matrix(NA,m,nph)
   #ph.pvals<-matrix(NA,m,nph)
   stat.names<-pval.names<-rep("",nph)
   if (!proj0)
   {
     for (i in 1:nph)
     {
       ph.var<-unlist(strsplit(as.character(ph.pattern$endpt.vars[i]),split=","))
       temp.ph<-ph.data[,ph.var]

       ph.stat<-as.character(ph.pattern$stat.func[i])
       ph.wgt<-ph.pattern$stat.coef[i]
       ph.call<-call(ph.stat,Y,temp.ph, strat)
       ph.val<-eval(ph.call)
       ph.stats[,i]<-ph.val
       stat.names[i]<-paste(unlist(c(unlist(ph.var),"stat")),collapse=".")
       res<-res+ph.wgt*ph.val
       sum.wgt<-sum.wgt+abs(ph.wgt)
     }
     res<-res/sum.wgt
   }
   if (proj0)
   { 
     for (i in 1:nph)
     {
       ph.var<-unlist(strsplit(as.character(ph.pattern$endpt.vars[i]),split=","))
       temp.ph<-ph.data[,ph.var]

       ph.stat<-as.character(ph.pattern$stat.func[i])
       ph.call<-call(ph.stat,Y,temp.ph, strat)
       ph.val<-eval(ph.call)
       ph.stats[,i]<-ph.val
       stat.names[i]<-paste(unlist(c(unlist(ph.var),"stat")),collapse=".")
       res<-res+ph.val^2
     }    
   }  
   colnames(ph.stats)<-stat.names
   stat<-cbind(ph.stats,PROMISE.stat=res)
   return(stat)
}

`jung.rstat` <-
function(x,time.cens,strat=NULL)
#  x: matrix of array data; row:probe, clumn: subject
#  time.cens: matrix of time and censor variable; first column is time and the second is censor variable
#  strat=NULL: stratification vector, entries for subjects, same order as columns of Y

{
  if (is.null (strat)) strat<-rep(1, dim(time.cens)[1])
	ustrat<-unique(strat)
	nstrat<-length(ustrat)
	stat.sum<-0
	sum.n<-0
	for (i in 1:nstrat)
	{
    this.strat<-(strat==ustrat[i])
		this.x<-x[,this.strat]
    this.n<-sum(this.strat)
    this.time.cens<-time.cens[this.strat,]
    
         time<-this.time.cens[,1]
         cens<-this.time.cens[,2]

         miss.tc<-is.na(time)|is.na(cens)
         time<-time[!miss.tc]
         cens<-cens[!miss.tc]
         this.x<-this.x[,!miss.tc]

         miss.this.x<-is.na(this.x)

       	 tR<-apply(this.x,1,rank)
         R<-t(tR)
         n<-length(time)
       	 Y<-matrix(0,n,n)
       	 for (i in 1:n)
         {
       	  	ind<-(time>=time[i])
       		  Y[i,]<-ind/sum(ind)
       	 }
         # Compute observed statistic
       	 I.Y<-diag(1,n)-Y
         s<-t(I.Y)%*%cens
         mn.s<-mean(s)
         sd.s<-sqrt(apply(s, 2, var))
         s.cnt<-(s-mn.s)/sd.s

         mn.R<-(n+1)/2
         sd.R<-sqrt(var(1:n))
         cnt.R<-(R-mn.R)/sd.R

         n<-rowSums(!miss.this.x)
         cnt.R[miss.this.x]<-0

         res<-as.vector((cnt.R%*%s.cnt)/n)
    
		stat.sum<-stat.sum+this.n*res
		sum.n<-sum.n+this.n
	}
  stat.sum<-as.vector(stat.sum/sum.n)
	return(stat.sum)
}

%\VignetteIndexEntry{An introduction to PROMISE}
%\VignetteDepends{Biobase,GSEABase}
%\VignetteKeywords{Microarray Association Pattern}
%\VignettePackage{PROMISE}
\documentclass[]{article}

\usepackage{times}
\usepackage{hyperref}

\newcommand{\Rpackage}[1]{{\textit{#1}}}

\title{An Introduction to \Rpackage{PROMISE}}
\author{Stan Pounds, Xueyuan Cao}
\date{June 24, 2014}

\begin{document}
\SweaveOpts{concordance=TRUE}

\maketitle

<<options,echo=FALSE>>=
options(width=60)
@ 

\section{Introduction}

PROMISE, PROjection onto the Most Interesting Statistical Evidence, is a general procedure to identify genomic features that 
exhibit a specific biologically interesting pattern of association with multiple phenotypic endpoint variables. Biological knowledge of the endpoint variables is used to 
define a vector that represents the biologically most interesting values for a set of association statistics. If prior biological knowledge of the endpoint variables is not available, a projection to point 0 is performed. The PROMISE performs one hypothesis test for each genomic feature, and is flexible to accommodate various types of endpoints. In this update, we also incorporte a fast permutation based on negative binomial sampling strategy, in which further permuation for a gene will not be performed for a gene with fixed number of success achieved.

In this document, we describe how to perform PROMISE procedure using hypothetical example data sets provided with the package. 

\section{Requirements}

The PROMISE package depends on {\em Biobase} and {\em GSEABase}. The understanding of {\em ExpressionSet} and {\em GeneSetCollection} is a prerequiste to 
perform the PROMISE procedure. Due to the internal handling of multiple endpoints, the consistency of {\em ExpressionSet} and {\em GeneSetCollection} is assumed.
The detailed requirements are illustrated below.

Load the PROMISE package and the example data sets: sampExprSet, sampGeneSet, and phPatt into R.

<<Load PROMISE package and data>>=
library(PROMISE)
data(sampExprSet)
data(sampGeneSet)
data(phPatt)
@

The {\em ExpressionSet} should contain at least two components: {\em exprs} (array data) and {\em phenoData} (endpoint data).
{\em exprs} is a data frame with column names representing the array identifiers (IDs) and row names representing the probe (genomic feature) IDs.
{\em phenoData} is an {\em AnnotatedDataFrame} with column names representing the endpoint variables and row names representing array.
The array IDs of {\em phenoData} and {\em exprs} should be matched.

<<Extract ArrayData and Endpoint Data from sampExprSet>>=
arrayData<-exprs(sampExprSet)
ptData<- pData(phenoData(sampExprSet))
head(arrayData[, 1:4])
head(ptData)
all(colnames(arrayData)==rownames(ptData))
@

The {\em GeneSetCollection} should be extractable in the following way. The probe IDs should be a subset of probe IDs of {\em exprs}. 
<<Extract sampGeneSet to a data frame>>= 
GS.data<-NULL
for (i in 1:length(sampGeneSet)){
    tt<-sampGeneSet[i][[1]]
    this.name<-unlist(geneIds(tt))
    this.set<-setName(tt)
    this.data<- cbind.data.frame(featureID=as.character(this.name),
             setID=rep(as.character(this.set), length(this.name)))
    GS.data<-rbind.data.frame(GS.data, this.data)
}
sum(!is.element(GS.data[,1], rownames(arrayData)))==0
@

The association pattern definition is critical. The prior biological knowledge is required to define the vector that represents the biologically most interesting values for statistics. If prior biological knowledge of the endpoint variables cannot be assumed, an arbitrary  \emph{stat.coef} can be used as it will be ignored with \emph{proj0=TRUE}. In this hypothetical example, we are interested in identifying genomic features that are positively associated with active drug level, negatively associated with minimum disease, and positively 
associated with survival. The three endpoints are represented in three rows as shown below:

<<Display phPatt>>=
phPatt
@

\section{PROMISE Analysis}
As mentioned in section 2, the {\em ExpressionSet} and pattern definition are required by PROMISE procedure. 
{\em GeneSetCollection} is required if a gene set enrichment analysis (GSEA) is to be performed within the PROMISE analysis.


The code below performs a PROMISE analysis without GSEA. As mentioned above, {\em GeneSetCollection} is not needed. The gene set result is NULL.
<<PROMISE without GSEA>>=
test1 <- PROMISE(exprSet=sampExprSet,
                 geneSet=NULL, 
                 promise.pattern=phPatt, 
                 strat.var=NULL,
                 proj0=FALSE,
                 nbperm=FALSE, 
                 max.ntail=10,
                 seed=13, 
                 nperms=100)
@
Gene level (genomic feature) result:
<<Gene Level Result>>=
gene.res<-test1$generes
head(gene.res)
@
Gene set level result:
<<Gene Set Result>>=
set.res<-test1$setres
head(set.res)
@

The code below performs a PROMISE analysis with GSEA and using fast permutation. As mentioned above, {\em GeneSetCollection} is required. sampGeneSet, a {\em GeneSetCollection}, is passed as an argument to PROMISE.
<<PROMISE with GSEA>>=
test2 <- PROMISE(exprSet=sampExprSet,
                 geneSet=sampGeneSet, 
                 promise.pattern=phPatt, 
                 strat.var=NULL,
                 proj0=FALSE,
                 nbperm=TRUE,
                 max.ntail=10,
                 seed=13, 
                 nperms=100)
@
Gene level (genomic feature) result:
<<Gene Level Result>>=
gene.res2<-test2$generes
head(gene.res2)
@
Gene set level result:
<<Gene Set Result>>=
set.res2<-test2$setres
head(set.res2)
@

The code below performs a PROMISE analysis with GSEA and using fast permutation without prior knowlege of the three endpoint variables (\emph{ proj0=TRUE}). As mentioned above, {\em GeneSetCollection} is required. sampGeneSet, a {\em GeneSetCollection}, is passed as an argument to PROMISE.
<<PROMISE with GSEA>>=
test3 <- PROMISE(exprSet=sampExprSet,
                 geneSet=sampGeneSet, 
                 promise.pattern=phPatt, 
                 strat.var=NULL,
                 proj0=TRUE,
                 nbperm=TRUE,
                 max.ntail=10,
                 seed=13, 
                 nperms=100)
@
Gene level (genomic feature) result:
<<Gene Level Result>>=
gene.res3<-test3$generes
head(gene.res3)
@
Gene set level result:
<<Gene Set Result>>=
set.res3<-test3$setres
head(set.res3)
@
\end{document}

\name{jung.rstat}
\alias{jung.rstat}
\Rdversion{3.1.0}
\title{Function to Compute Jung's Statistics }
\description{
  Compute statistic that measures the correlation of many continuous variables with a censored time-to-event variable
}
\usage{
jung.rstat(x, time.cens, strat = NULL)
}

\arguments{
  \item{x}{ 
  a data frame with row corresponding to probe set and column corresponding
  to subjects, the order of columns (subjects) should match the order of rows in \emph{time.cens}.
  }
  \item{time.cens}{ 
  a data frame with number of row equal to number of column in \emph{x}.
  It contains two columns with first for time and second for censor
  (1 = event, 0 = censored).   
  }
  \item{strat}{ 
  a vector of stratum to calculate stratified r-type association statistics, default = NULL.
  }
}

\value{
Returns a vector of Jun's r-type association statistics.
}
\references{ 
 Jung SH, Owzar K, and George SL (2005) A multiple testing procedure to 
 associate gene expression levels with survival. Stat Med 24:3077-88
 }
\author{ 
Stan Pounds \email{stanley.pounds@stjude.org}; Xueyuan Cao \email{xueyuan.cao@stjude.org}
}
\note{
The order of subjects in \emph{x} (column), \emph{time.cens}, and \emph{strat} should all match. 
The original statistic proposed by Jung, Owzar, and George can be written as a dot-product.  
The statistic returned by this routine is expressed in the form of a correlation statistic by dividing the dot product by the square root of the lengths of the two vectors in the numerator.
}
\seealso{
\code{\link{PROMISE}}
}
\examples{
## load sampExprSet.
data(sampExprSet)

## extract expression matrix from sampExprSet
Y <- exprs(sampExprSet)

## extract end point data from sampExprSet
time.cens <-pData(phenoData(sampExprSet))[, 3:4] 
strat <- pData(phenoData(sampExprSet))$strat

## compute Jung's r-type association statistics
jungstat <- jung.rstat(Y, time.cens, strat = strat)
}
\keyword{survival}

\name{spearman.rstat}
\Rdversion{3.1.0}
\alias{spearman.rstat}
\title{
Function to Calculate Spearman Correlation Statistics
}
\description{
A function to calculate Spearman rank correlation of each gene in an array data with a continuous variable
}
\usage{
spearman.rstat(Y, x, strat = NULL)
}
\arguments{
  \item{Y}{
a numeric data frame. Each row gives values of one genomic variable.
}
  \item{x}{
a vector of continuous variable.
}
  \item{strat}{
a vector of stratum to calculate stratified correlation statistics, default = NULL.
}
}
\value{
Return a vector of Spearman rank correlation statistics.
}
\references{
Spearman C. (1904) The proof and measurement of association between two things. Amer. J. Psychol. 15: 72-101
}
\author{   
Stan Pounds \email{stanley.pounds@stjude.org}; Xueyuan Cao \email{xueyuan.cao@stjude.org}
}
\seealso{
\code{\link{PROMISE}}
}
\examples{
## load sampExprSet.
data(sampExprSet)

## extract expression matrix from sampExprSet
Y <- exprs(sampExprSet)

## extract end point data from sampExprSet
x <- pData(phenoData(sampExprSet))$drugLevel
strat <- pData(phenoData(sampExprSet))$strat

## Calculte Spearman correlation statistics
test <- spearman.rstat(Y, x, strat = strat)
}
\keyword{univar}

\name{PROMISE}
\alias{PROMISE}
\Rdversion{3.1.0}
\title{
PRojection onto the Most Interesting Statistical Evidence
}
\description{
Perform permutation-based test to identify genes with expression levels having a specific biologically interesting pattern of associations
with multiple endpoint variables
}
\usage{
PROMISE(exprSet, geneSet=NULL, promise.pattern, strat.var=NULL, proj0=FALSE, seed=13, nbperm=FALSE, max.ntail=100,
        nperms=10000)
}
\arguments{
  \item{exprSet}{
   an ExpressionSet class contains minimum of \emph{exprs} (\emph{expression matrix}) and \emph{phenoData} (\emph{AnnotatedDataFrame of end point data}). 
   Please refer to \emph{Biobase} for details on how to create such an ExpressionSet.}
  \item{geneSet}{
   a GeneSetCollection class with minimum of setName and geneIDs for each GeneSet. 
   Please refer to \emph{GSEABase} for how to create such a GeneSetCollection class. The default is NULL which will perform no gene set enrichment analysis.}  
  \item{promise.pattern}{
  a data frame defining the association pattern of interest. The column names must be \emph{stat.coef}, \emph{stat.func}, and \emph{endpt.vars}.
  The \emph{stat.coef} column gives the coefficients for combining the statistics of association of genomic variables with individual endpoint variable into the ultimate PROMISE statistic. If \emph{proj0=TRUE}, the \emph{stat.coeff} is ignored.  
  The \emph{stat.func} column gives the name of the R routine that computes the test statistic of association of the endpoint variables.
  Two R routines (\emph{jung.rstat} and \emph{spearman.rstat})are provided. Users can provide their own routine accordingly. 
  The \emph{endpt.vars} column gives the name(s) of variable(s) in the endpoint data file needed to compute each term of 
  the PROMISE statistic. A common without a space should be used to separate multiple variables that correspond to the same term in the association pattern definition.
  } 
  \item{strat.var}{
  the name or numeric value of stratum variable in \emph{exprSet} for stratified analysis. The default is NULL which performs an unstratified analysis.
  }
  \item{proj0}{
  indicator of whether projection to 0 is performed. It takes two valid values: TRUE or FALSE. If \emph{proj0=TRUE}, PROMISE statistics is the sum of squares of individual statitics and the \emph{stat.coeff} in \emph{promise.pattern} is ignored. The default is FALSE.
  }
  \item{seed}{
  initial seed of random number generator. The default is 13.}
  \item{nbperm}{
  indicator of fast permuation using negative binomial strategy, taking two valid values: FALSE or TRUE. The default is FALSE.}
  \item{max.ntail}{
  number of sucess if \emph{nbperm = T}. Further permutation will not be performed for gene(s) or gene set(s) which \emph{max.ntail} permutated statistics are greater or equal to the observed statistics, The default is 100}
  \item{nperms}{
  number of permutations. The default is 10,000.}
}
\value{
  \item{$generes }{individual genes' test statistics and p-values for each individual endpoint 
  and PROMISE analysis. If \emph{nbperm=T}, the last column contains number of permuations for each gene.
  }
  \item{$setres}{gene set's test statistics and p-values for each individual endpoint 
  and PROMISE analysis. If \emph{nbperm=T}, the last column contains number of permuations for gene set. If \emph{geneSet} is NULL, the value of this component is also \emph{NULL}. 
  }
}
\references{
Pounds S, Cheng C, Cao X, Crews KR, Plunkett W, Gandhi V, Rubnitz J, Ribeiro RC, Downing JR, and Lamba J (2009) 
PROMISE: a tool to identify genomic features with a specific biologically interesting pattern of associations 
with multiple endpoint variables. Bioinformatics 25: 2013-2019
}
\author{
Stan Pounds \email{stanley.pounds@stjude.org}; Xueyuan Cao \email{xueyuan.cao@stjude.org}
 }
\seealso{\code{\link{jung.rstat}} \code{\link{avg.abs.genestat}} 
\code{\link{promise.genestat}}  \code{\link{spearman.rstat}}
\code{\link{promise.pattern}}
}
\examples{
## load sampExprSet, sampGeneSet, phPatt.
data(sampExprSet)
data(sampGeneSet)
data(phPatt)

## Perform PROMISE procedure without GSEA
test1 <- PROMISE(exprSet=sampExprSet,
                 geneSet=NULL, 
                 promise.pattern=phPatt, 
                 strat.var=NULL,
                 proj0=FALSE,
                 nbperm=FALSE, 
                 max.ntail=10,
                 seed=13, 
                 nperms=100)
             
## Perform PROMISE procedure with GSEA and using fast permuation              
test2 <- PROMISE(exprSet=sampExprSet,
                 geneSet=sampGeneSet, 
                 promise.pattern=phPatt, 
                 strat.var=NULL,
                 proj0=TRUE,
                 nbperm=TRUE, 
                 max.ntail=10,
                 seed=13, 
                 nperms=100)
}        
\keyword{multivariate}

\name{sampGeneSet}
\alias{sampGeneSet}
\title{An Example Gene Set Collection}
\description{  
    This hypothetical gene set \emph{sampGeneSet} belongs to a \emph{GeneSetCollection} class. 
    It contains 10 gene sets (\emph{GeneSet} class). 
}
\usage{
data(sampGeneSet)
}
\keyword{misc}

\name{avg.abs.genestat}
\alias{avg.abs.genestat}
\Rdversion{3.1.0}
\title{
Function to Compute Gene Set Statistics
}
\description{
A function to calculate the mean of absolute values of statistics based on a gene set definition
}
\usage{
avg.abs.genestat(gene.res, probes, GS.data)
}
\arguments{
  \item{gene.res}{
  a data frame. Each row gives test statistics for a genomic variable. Each column corresponds to
  an endpoint variable.
  }
  \item{probes}{
  a vector that links the \emph{gene.res} to \emph{GS.data}.                                         
  }
  \item{GS.data}{
  a data frame with first column for probe set identifier and second column for gene set identifier. 
  Each row assigns a probe set to a gene set. Each probe set may be assigned to multiple gene sets or no gene set at all.
  }
}
\value{
 Return a matrix of statistics. Each row gives the mean absolute value of test statistics of genes belonging to a gene set. 
 The columns are same as in \emph{gene.res}.
 }
\references{
Goeman JJ  and Buhlmann P (2007) Analyzing gene expression data in terms of gene sets: methodological issues. 
Bioinformatics  23: 980-987.
}
\author{
Stan Pounds \email{stanley.pounds@stjude.org}; Xueyuan Cao \email{xueyuan.cao@stjude.org}
 }
\seealso{
\code{\link{PROMISE}}
}
\note{
A function internally called by \emph{PROMISE}. 
}
\examples{

## load sampExprSet sampGeneSet.
data(sampExprSet)
data(sampGeneSet)

## extract expression matrix from sampExprSet
Y <- exprs(sampExprSet)
probes <- rownames(Y)

## convert sampGeneSet to a data frame
GS.data <- NULL
for (i in 1:length(sampGeneSet)){
    tt <- sampGeneSet[i][[1]]
    this.name <- unlist(geneIds(tt))
    this.set <- setName(tt)
    GS.data <- rbind.data.frame(GS.data,
                cbind.data.frame(featureID=as.character(this.name),
                                 setID=rep(as.character(this.set),
                                 length(this.name))))
}

## Calculate the mean of absolute values of statistics
## This is only a demo, probe expression values are used 
##in stead of statistics 
test <- avg.abs.genestat(Y, probes, GS.data)
}
\keyword{multivariate}

\name{PROMISE-package}
\alias{PROMISE-package}
\Rdversion{3.1.0}
\docType{package}
\title{
PRojection Onto the Most Interesting Statistical Evidence
}
\description{
a tool to identify genomic geatures with a
specific biologically interesting pattern of associations
with multiple endpoint variables
}
\details{
\tabular{ll}{
Package: \tab PROMISE\cr
Type: \tab Package\cr
Version: \tab 1.17.0\cr
Date: \tab 2014-6-24\cr
License: \tab GPL (>=2)\cr
LazyLoad: \tab yes\cr
}
The PROMISE (PRojection Onto the Most Interesting Statistical Evidence) is performed by calling function PROMISE.
The array data and endpoint data are passed through an ExpressionSet; the gene set definition is passed through a GeneSetCollection, 
and PROMISE definition is passed through a data frame. \emph{promise.genestat} and \emph{avg.abs.genestat} are called internally by PROMISE.
Two R routines for calculating association statistics with individual endpoint variable(\emph{jung.rstat} and \emph{spearman.rstat}) are 
provided in this version. Users could provide their own R routines written in a similar fashion. 
}
\author{
Stan Pounds \email{stanley.pounds@stjude.org}; Xueyuan Cao \email{xueyuan.cao@stjude.org}

Maintainer: Stan Pound \email{stanley.pounds@stjude.org}; Xueyuan Cao \email{xueyuan.cao@stjude.org}
}
\references{
Jung, S-H, Owzar K, and Goerge SL (2005) A multiple testing procedure to associate gene expression levels with survival.  
Biostatistics 24: 3077-3088.

Goeman JJ  and Buhlmann P (2007) Analyzing gene expression data in terms of gene sets: methodological issues. 
Bioinformatics  23: 980-987.

Pounds S, Cheng C, Cao X, Crews KR, Plunkett W, Gandhi V, Rubnitz J, Ribeiro RC, Downing JR, and Lamba J (2009) 
PROMISE: a tool to identify genomic features with a specific biologically interesting pattern of associations 
with multiple endpoint variables. Bioinformatics 25: 2013-2019
}
\keyword{ package }
\examples{
## load sampExprSet, sampGeneSet, phPatt.
data(sampExprSet)
data(sampGeneSet)
data(phPatt)

## Perform PROMISE procedure without GSEA
test1 <- PROMISE(exprSet=sampExprSet,
                 geneSet=NULL, 
                 promise.pattern=phPatt, 
                 strat.var=NULL,
                 proj0=FALSE,
                 nbperm=FALSE, 
                 max.ntail=10,
                 seed=13, 
                 nperms=100)
             
## Perform PROMISE procedure with GSEA and using fast permuation              
test2 <- PROMISE(exprSet=sampExprSet,
                 geneSet=sampGeneSet, 
                 promise.pattern=phPatt, 
                 strat.var=NULL,
                 proj0=TRUE,
                 nbperm=TRUE, 
                 max.ntail=10,
                 seed=13, 
                 nperms=100)
}

\name{promise.genestat}
\alias{promise.genestat}
\Rdversion{3.1.0}
\title{
Function to Calculate PROMISE Statistics
}
\description{
a function to calculate individual gene and PROMISE statistics for a defined pattern of association
}
\usage{
promise.genestat(Y, ph.data, ph.pattern, strat = NULL, proj0=FALSE)
}
\arguments{
  \item{Y}{
  a data frame with row corresponding to probe set and column corresponding
  to subjects, the order of column should match order of row in \emph{ph.data}.
  }
  \item{ph.data}{
  a data frame with rows corresponding to subjects and columns 
  corresponding to endpoint variables.
  }
  \item{ph.pattern}{
  a data frame with column headers: \emph{stat.coef}, \emph{stat.func}, \emph{endpt.vars}. The \emph{stat.coeff} column gives 
  the coefficients for combining the statistics of association of genomic variable with individual endpoint 
  variable into the ultimate PROMISE statistic. If \emph{proj0=TRUE}, the \emph{stat.coeff} is ignored. The \emph{stat.func} column gives the name of the R routine that 
  computes the test statistic of association of the end point variables.
  \emph{jung.rstat} and \emph{spearman.rstat} are provided. Users can provide their own routines accordingly. 
  The \emph{endpt.vars} column gives the name(s) of variable(s) in  
  \emph{ph.data} needed to compute each term of the PROMISE statistic. A \bold{comma} without a space should be used 
  to separate multiple variables that correspond to the same term in the association pattern definition.       
  }
  \item{strat}{
  a vector of stratum to calculate stratified statistics. The default is NULL.}
  \item{proj0}{
  indicator of whether projection to 0 is performed. It takes two valid values: TRUE or FALSE. If proj0=TRUE, PROMISE statistics is the sum of squares of individual statitics. The default is FALSE.
  }
}

\value{
 a matrix of statistics. Each row gives gene's statistics of each individual endpoint and the PROMISE statistics defined in \emph{ph.pattern}.
}
\references{
Pounds S, Cheng C, Cao X, Crews KR, Plunkett W, Gandhi V, Rubnitz J, Ribeiro RC, Downing JR, and Lamba J (2009) 
PROMISE: a tool to identify genomic features with a specific biologically interesting pattern of associations 
with multiple endpoint variables. Bioinformatics 25: 2013-2019
}
\author{
Stan Pounds \email{stanley.pounds@stjude.org}; Xueyuan Cao \email{xueyuan.cao@stjude.org}
}
\note{
a function internally called by PROMISE.
}
\seealso{\code{\link{PROMISE}}}
\examples{
## load sampExprSet, phPatt.
data(sampExprSet)
data(phPatt)

Y <- exprs(sampExprSet)
ph.data <- pData(phenoData(sampExprSet))

test <- promise.genestat(Y, ph.data, phPatt, strat=ph.data[, 5])
test2 <- promise.genestat(Y, ph.data, phPatt, strat=ph.data[, 5], proj0=TRUE)
}
\keyword{multivariate}

\name{promise.pattern}
\alias{promise.pattern}
\title{
PROMISE pattern
}
\description{
PROMISE pattern is a data frame of association pattern definition, consisting of three columns. 
}
\format{
PROMISE pattern:  The column names must be \emph{stat.coef}, \emph{stat.func}, and \emph{endpt.vars}.
 
\emph{stat.coef} column gives the coefficients for combining the statistics of association of genomic variable with individual endpoint variable into the ultimate PROMISE statistic.
 
\emph{stat.func} column gives the name of the R routine that computes the test statistic of association of the endpoint variables.
  Two R routines (\emph{jung.rstat} and \emph{spearman.rstat})are provided in current release. Users can provide their own routine accordingly.
   
\emph{endpt.vars} column gives the name(s) of variable(s) in the endpoint data frame needed to compute each term of 
  the PROMISE statistic. If more than one variables involve in one term, they should be separated by a \bold{comma} without space. 
}

\references{
Pounds S, Cheng C, Cao X, Crews KR, Plunkett W, Gandhi V, Rubnitz J, Ribeiro RC, Downing JR, and Lamba J (2009) 
PROMISE: a tool to identify genomic features with a specific biologically interesting pattern of associations 
with multiple endpoint variables. Bioinformatics 25: 2013-2019
}
\author{
Stan Pounds \email{stanley.pounds@stjude.org}; Xueyuan Cao \email{xueyuan.cao@stjude.org}
 }
 \seealso{\code{\link{PROMISE}} 
}

\keyword{misc}
    

\name{sampExprSet}
\alias{sampExprSet}
\title{An Example Expression Set}
\description{  
    This hypothetical expression set \emph{sampExpSet} belongs to an \emph{ExpressionSet} class. 
    It contains 100 genomic features (probe_1 to probe_100) for 50 subjects (array_1 to array_50) 
    and phenotype data of drugLevel, residualDisease, obsTime, obsCensor and strat. The expression values can be accessed by \emph{exprs(sampExprSet)}.
    The phenotype data can be accessed by \emph{pData(phenoData(sampExprSet))}
}
\usage{
data(sampExprSet)
}
\keyword{misc}

\name{phPatt}
\alias{phPatt}
\title{Phenotype Pattern Definition Set}
\description{  
    This hypothetical phenotype pattern definition set \emph{phPatt} has three columns: stat.coef, stat.func, and endpt.vars. 
    It defines an associatin pattern for three phenotypes.
}
\usage{
data(phPatt)
}
\keyword{misc}
